import os
import json
import requests
from tqdm import tqdm
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from nanonets.model import Model
from nanonets.utils import *

class ImageClassification(Model):

	def __init__(self, api_key, categories, model_id=None):
		super().__init__(api_key=api_key, model_type='ImageCategorization')
		if not model_id:
			self._create_model(categories)
		else:
			self.api_key = api_key
			self.categories = categories
			self.model_id = model_id

	def train_dict_to_category_dict(self, training_dict):
		"""
		function to split the training_dict into separate dicts for each category

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file paths and values as the corresponding labels. 

		Returns
		-------
		Dict with keys as categories of the model and values as List[str] of image paths 
		"""

		train_dicts = {}
		for category in self.categories:
			train_dicts[category] = []
		for file in training_dict:
			category = training_dict[file]
			train_dicts[category].append(file)
		return train_dicts

	def upload_image_file(self, img_path, label):
		"""
		function to upload a single file and it's annotation for training 

		Parameters
		----------
		img_path: str
		    path to the image we want to upload

		label: str
			label for the same image

		Returns
		-------
		server response for the request for uploading urls. 
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/UploadFile/'
		params = {'file': open(img_path, 'rb'),
				  'modelId': ('', self.model_id),
				  'category': ('', label)}
		response = requests.post(url,
					files=params, 
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''))
		return response

	def upload_image_files(self, training_dict, batch_size=20):
		"""
		function to upload multiple files and their annotations 

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file paths and values as the corresponding labels 

		batch_size (optional): int
			number of images to be uploaded per API request

		Returns
		-------
		a list of responses for each batch of images uploaded
		"""

		train_dicts = self.train_dict_to_category_dict(training_dict)
		responses = []
		for category in self.categories:
			print('Uploading for category: ', category)
			batch_nb = 1
			files = train_dicts[category]
			if len(files)%batch_size == 0:
				total_batches = int(len(files)/batch_size)
			else:
				total_batches = int(int(len(files)/batch_size) + 1)
			url = self.host + self.model_type + '/UploadFile/?modelId=%s&category=%s'%(self.model_id, category)		
			while len(train_dicts[category]) > 0:
				multiple_files = []
				batch_images, train_dicts[category] = train_dicts[category][:batch_size], train_dicts[category][batch_size:]
				print('Batch {}/{} of images'.format(batch_nb, total_batches))				
				for image in tqdm(batch_images):
					image_name = image.split('/')[-1]
					multiple_files.append(('file', (image_name, open(image, 'rb'), 'image/jpeg')))
				response = requests.post(url, 
							 auth=requests.auth.HTTPBasicAuth(self.api_key, ''), 
							 files=multiple_files)
				responses.append(response)
				batch_nb+=1
		return responses

	def upload_image_url(self, image_url, label):
		"""
		function to upload image given by their url to a model that has been created

		Parameters
		----------
		image_url: str
		    path to the image we want to upload

		label: str
			label for the same image

		Returns
		-------
		server response for the request for uploading url
		"""

		url = self.host + self.model_type + '/UploadUrls/'
		data = {'urls': [image_url],
			'modelId': self.model_id,
			'category': label}
		headers = { 'accept': 'application/x-www-form-urlencoded'}
		response = requests.post(url,
								 headers=headers,
								 auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
								 data=data)
		return response

	def upload_image_urls(self, training_dict):
		"""
		function to upload images given by their urls to a model that has been created

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file urls and values as the corresponding labels 

		Returns
		-------
		list of server responses for the request for uploading urls for each category
		"""

		url = self.host + self.model_type + '/UploadUrls/'
		train_dicts = self.train_dict_to_category_dict(training_dict)
		responses = []
		for category in self.categories:
			data = {'urls': train_dicts[category],
				'modelId': self.model_id,
				'category': category}
			headers = { 'accept': 'application/x-www-form-urlencoded'}
			response = requests.post(url,
						 headers=headers,
						 auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
						 data=data)
			responses.append(response)
		return responses

	def train(self, training_dict, data_path_type='files', batch_size=20):
		"""
		function to upload a single file and it's labels and initiate the training. 

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file urls/paths and values as the corresponding labels

		data_path_type: str
			Can take values 'files' or 'urls'

		Returns
		-------
		server response for the request for intialising training
		"""
		if data_path_type == 'files':		
			self.upload_image_files(training_dict, batch_size=batch_size)
		elif data_path_type == 'urls':
			self.upload_image_urls(training_dict)
		url = self.host + self.model_type + '/Train/'
		params = {'modelId': self.model_id}
		response = requests.request("POST", url,
					    params=params,
					    auth=requests.auth.HTTPBasicAuth(self.api_key, ''))
		return response

	def predict_for_file(self, file_path):
		"""
		function to get prediction for a single image file

		Parameters
		----------
		file_path: 
			path of the image you want to get predictions for

		Returns
		-------
		JSON repsonse of the prediction 
		"""

		url = self.host + self.model_type + '/LabelFile/?modelId=%s'%(self.model_id)
		params = {'file': open(file_path, 'rb')}
		response = requests.post(url,
					 auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					 files=params) 
		result = read_prediction_response([response], self.model_type)
		return result

	def predict_for_url(self, image_url):
		"""
		function to get prediction for a single image file

		Parameters
		----------
		image_url: 
			url of the image you want to get predictions for

		Returns
		-------
		JSON repsonse of the prediction 
		"""

		url = self.host + self.model_type + '/LabelUrls/'
		headers = {'accept': 'application/x-www-form-urlencoded'}
		params = {'modelId': self.model_id, 'urls': [image_url]}
		response = requests.post(url,
					headers=headers,
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					data=params) 
		result = read_prediction_response([response], self.model_type)
		return result

	def predict_for_files(self, files, batch_size=20):
		"""
		function to get predictions for multiple image files 

		Parameters
		----------
		files: List[str]
		    List of image file paths

		batch_size (optional): int
			number of images to be uploaded per API request

		Returns
		-------
		Dict with file names as keys and the predictions as values
		"""

		url = self.host + self.model_type + '/LabelFile/?modelId=%s'%(self.model_id)
		retries = Retry(total=5, backoff_factor=1, status_forcelist=[ 502, 503, 504 ])
		adapter = HTTPAdapter(max_retries=retries)

		session = requests.Session()
		session.mount(url, adapter)

		responses = []
		batch_nb = 1
		if len(files)%batch_size == 0:
			total_batches = int(len(files)/batch_size)
		else:
			total_batches = int(int(len(files)/batch_size) + 1)
		while len(files) > 0:
			print('Batch {}/{} of images'.format(batch_nb, total_batches))				
			batch, files = files[:batch_size], files[batch_size:]
			multiple_files = []
			for image in tqdm(batch):
				image_name = image.split('/')[-1]
				multiple_files.append(('file', (image_name, open(image, 'rb'), 'image/jpeg')))
			response = session.post(url, 
						auth= requests.auth.HTTPBasicAuth(self.api_key, ''), 	
						files=multiple_files)
			responses.append(response)
			batch_nb+=1
		result = read_prediction_response(responses, self.model_type)
		return result

	def predict_for_urls(self, urls):
		"""
		function to get prediction for several images using their urls

		Parameters
		----------
		urls: List[str]
			list of urls of the images 

		Returns
		-------
		JSON repsonse of the predictions
		"""

		url = self.host + self.model_type + '/LabelUrls/'
		headers = {'accept': 'application/x-www-form-urlencoded'}
		params = {'modelId': self.model_id, 'urls': urls}
		response = requests.post(url,
					headers=headers,
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					data=params) 
		result = read_prediction_response([response], self.model_type)
		return result



import os
import json
import requests
from tqdm import tqdm
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from nanonets.model import Model
from nanonets.utils import *


class ObjectDetection(Model):

	def __init__(self, api_key, categories, model_id=None):
		super().__init__(api_key=api_key, model_type='ObjectDetection')
		if not model_id:
			self._create_model(categories)
		else:
			self.api_key = api_key
			self.categories = categories
			self.model_id = model_id


	def upload_image_file(self, img_path, annotation_path):
		"""
		function to upload a single file and it's annotation for training 

		Parameters
		----------
		img_path: str
		    path to the image we want to upload

		annotation_path: str
			path to the annotations for the same image

		Returns
		-------
		server response for the request for uploading image and annotations
		"""

		processed = [process_annotations(annotation_path)]
		annotation = json.dumps(processed)
		return self._upload_image_file(img_path, annotation)

	def upload_image_files(self, training_dict, batch_size=20):
		"""
		function to upload multiple files and their annotations 

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file paths and values as the corresponding annotation 
		    paths. Annotations can be XML or JSON

		batch_size (optional): int
			number of images to be uploaded per API request

		Returns
		-------
		a list of responses for each batch of images uploaded
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/UploadFile/'
		files = list(training_dict.keys())
		batch_nb = 1
		if len(files)%batch_size == 0:
			total_batches = int(len(files)/batch_size)
		else:
			total_batches = int(int(len(files)/batch_size) + 1)
		responses = []
		while len(files) > 0:
			batch_files = []
			batch_data = []
			batch, files = files[:batch_size], files[batch_size:]
			print('Batch {}/{} of images'.format(batch_nb, total_batches))				
			for file in tqdm(batch):
				img_name = file.split('/')[-1]
				batch_data.append(process_annotations(img_name, training_dict[file]))
				batch_files.append(('file', (img_name, open(file, 'rb'), 'image/jpeg')))
			batch_files.append(('data', ('', json.dumps(batch_data))))
			response = requests.post(url, 
						 auth= requests.auth.HTTPBasicAuth(self.api_key, ''), 
						 files=batch_files)
			responses.append(response)
			batch_nb+=1
		return responses

	### UPLOAD IMAGES AND ANNOTATE THEM ON THE PLATFORM.
	def upload_image_url(self, image_url, annotation_path):
		pass

	### UPLOAD IMAGES AND ANNOTATE THEM ON THE PLATFORM.
	def upload_image_urls(self, training_dict):
		pass

	### FIGURE OUT ADDING URLS
	def train(self, training_dict, batch_size=20):
		"""
		function to upload a image files and their annotations and initialize the training 

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file paths and values as the corresponding annotation 
		    paths. Annotations can be XML or JSON

		Returns
		-------
		server response for the request for unplading images and initializing training
		"""

		print("Uploading images ...")
		self.upload_image_files(training_dict, batch_size=batch_size)
		print("Starting training ...")
		return self._train()

	def predict_for_file(self, file_path):
		"""
		function to get prediction for a single image file

		Parameters
		----------
		file_path: str
			path of the image you want to get predictions for

		Returns
		-------
		JSON repsonse of the prediction 
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/LabelFile/'
		params = {'file': open(file_path, 'rb')}
		response = requests.post(url,
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					files=params) 
		result = read_prediction_response([response], self.model_type)
		return result

	def predict_for_url(self, image_url):
		"""
		function to get prediction for a single image url

		Parameters
		----------
		image_url: str
			url of the image you want to get predictions for

		Returns
		-------
		JSON repsonse of the prediction 
		"""

		response = self._predict_urls([image_url])
		result = read_prediction_response([response], self.model_type)
		return result

	def predict_for_files(self, files, batch_size=20):
		"""
		function to get predictions for multiple files 

		Parameters
		----------
		files: List[str]
		    List of image file paths

		batch_size (optional): int
			number of images to be uploaded per API request

		Returns
		-------
		Dict with file names as keys and the predictions as values
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/LabelFile/'
		retries = Retry(total=5, backoff_factor=1, status_forcelist=[ 502, 503, 504 ])
		adapter = HTTPAdapter(max_retries=retries)

		session = requests.Session()
		session.mount(url, adapter)

		responses = []
		batch_nb = 1
		if len(files)%batch_size == 0:
			total_batches = int(len(files)/batch_size)
		else:
			total_batches = int(int(len(files)/batch_size) + 1)
		while len(files) > 0:
			batch, files = files[:batch_size], files[batch_size:]
			batch_files = []
			batch_data = []
			print('Batch {}/{} of images'.format(batch_nb, total_batches))				
			for file in tqdm(batch):
				img_name = file.split('/')[-1]
				batch_data.append({'filename': img_name})
				batch_files.append(('file', (img_name, open(file, 'rb'), 'image/jpeg')))
			batch_files.append(('data', ('', json.dumps(batch_data))))
			response = session.post(url, 
						auth=requests.auth.HTTPBasicAuth(self.api_key, ''), 
						files=batch_files)
			responses.append(response)
			batch_nb+=1
		result = read_prediction_response(responses, self.model_type)
		return result

	def predict_for_urls(self, urls):
		"""
		function to get prediction for several images using their urls

		Parameters
		----------
		urls: List[str]
			list of urls of the images you want to get predictions for
		Returns
		-------
		JSON repsonse of the predictions
		"""

		response = self._predict_urls(urls)
		result = read_prediction_response([response], self.model_type)
		return result		



import os
import json
import requests
from tqdm import tqdm
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from nanonets.model import Model
from nanonets.utils import *

class MultilabelClassification(Model):

	def __init__(self, api_key, categories, model_id=None):
		super().__init__(api_key=api_key, model_type='MultiLabelClassification')
		if not model_id:
			self._create_model(categories)
		else:
			self.api_key = api_key
			self.categories = categories
			self.model_id = model_id
	
	def upload_image_file(self, img_path, label_list):
		"""
		function to upload a single file and its labels for training to a model that 
		has been created.

		Parameters
		----------
		img_path: str
		    path to the image we want to upload

		label_list: List[str]
			list of all labels associated with the image

		Returns
		-------
		server response for the request for uploading image file and labels
		"""

		return self._upload_image_file(self, img_path, label_list)

	def upload_image_files(self, training_dict, batch_size=20):		
		"""
		function to upload multiple files and their labels to a model that has been 
		created.

		Parameters
		----------
		training_dict: Dict[str: List[str]]
		    Dict with keys as image file paths and values as the corresponding list of 
		    labels

		batch_size (optional): int
			number of images to be uploaded per API request

		Returns
		-------
		a list of responses for each batch of images uploaded
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/UploadFiles/'
		files = list(training_dict.keys())
		batch_nb = 1
		if len(files)%batch_size == 0:
			total_batches = int(len(files)/batch_size)
		else:
			total_batches = int(int(len(files)/batch_size) + 1)
		responses = []
		while len(files) > 0:
			batch_files = []
			batch_data = []
			batch, files = files[:batch_size], files[batch_size:]
			print('Batch {}/{} of images'.format(batch_nb, total_batches))				
			for file in tqdm(batch):
				img_name = file.split('/')[-1]
				batch_data.append({'filename': img_name, 'categories': training_dict[file]})
				batch_files.append(('files', (img_name, open(file, 'rb'), 'image/jpeg')))
			batch_files.append(('data', ('', json.dumps(batch_data))))
			response = requests.post(url, 
						 auth=requests.auth.HTTPBasicAuth(self.api_key, ''), 
						 files=batch_files)
			responses.append(response)
			batch_nb+=1
		return responses

	def upload_image_url(self, image_url, label_list):
		"""
		function to upload image given by their url to a model that has been created

		Parameters
		----------
		image_url: str
		    path to the image we want to upload

		label: List[str]
			list of labels for the same image

		Returns
		-------
		server response for the request for uploading image using url
		"""

		url = self.host + self.model_type + '/UploadUrls/'
		data = {'urls': [image_url],
			'modelId': self.model_id,
			'categories': label_list}
		headers = { 'accept': 'application/x-www-form-urlencoded'}
		response = requests.post(url,
					 headers=headers,
					 auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					 data=data)
		return response

	def upload_image_urls(self, training_dict):
		"""
		function to upload images given by their urls to a model that has been created

		Parameters
		----------
		training_dict: Dict[str: str]
		    Dict with keys as image file urls and values as the corresponding labels

		Returns
		-------
		server response for the request for uploading urls of images and their labels
		"""

		url = self.host + self.model_type + '/UploadUrls/'
		data = {'urls': list(training_dict.keys()),
			'modelId': self.model_id,
			'categories': list(training_dict.values())}
		headers = { 'accept': 'application/x-www-form-urlencoded'}
		response = requests.post(url,
					 headers=headers,
					 auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					 data=data)
		return response

	def train(self, training_dict, data_path_type='files', batch_size=20):
		"""
		function to upload image files/urls with their labels and initialise model training 

		Parameters
		----------
		training_dict: Dict[str: List[str]]
		    Dict with keys as image file urls/paths and values as the corresponding list of 
		    labels

		data_path_type: str
			Can take values 'files' or 'urls'

		Returns
		-------
		server response for the request for uploading images
		"""
		if data_path_type == 'files':		
			self.upload_image_files(training_dict, batch_size=batch_size)
		elif data_path_type == 'urls':
			self.upload_image_urls(training_dict)
		return self._train()

	def predict_for_file(self, file_path):
		"""
		function to get prediction for a single image file

		Parameters
		----------
		file_path: str
			path of the image you want to get predictions for

		Returns
		-------
		JSON repsonse of the prediction 
		"""
		url = self.host + self.model_type + '/Model/' + self.model_id + '/LabelFiles/'
		params = {'files': open(file_path, 'rb'), 'modelId': ('', self.model_id)}
		response = requests.post(url,
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					files=params) 
		result = read_prediction_response([response], self.model_type)
		return result

	def predict_for_url(self, image_url):

		"""
		function to get prediction for a single image file

		Parameters
		----------
		image_url: str
			url of the image you want to get predictions for

		Returns
		-------
		JSON repsonse of the prediction 
		"""

		response = self._predict_urls([image_url])
		result = read_prediction_response([response], self.model_type)
		return result

	def predict_for_files(self, files, batch_size=20):
		"""
		function to get predictions for multiple files 

		Parameters
		----------
		files: List[str]
		    List of image file paths

		batch_size (optional): int
			number of images to be uploaded per API request

		Returns
		-------
		Dict with file names as keys and the predictions as values
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/LabelFiles/'
		retries = Retry(total=5, backoff_factor=1, status_forcelist=[ 502, 503, 504 ])
		adapter = HTTPAdapter(max_retries=retries)

		session = requests.Session()
		session.mount(url, adapter)

		responses = []
		batch_nb = 1
		if len(files)%batch_size == 0:
			total_batches = int(len(files)/batch_size)
		else:
			total_batches = int(int(len(files)/batch_size) + 1)
		while len(files) > 0:
			batch, files = files[:batch_size], files[batch_size:]
			batch_files = []
			batch_data = []
			print('Batch {}/{} of images'.format(batch_nb, total_batches))				
			for file in tqdm(batch):
				img_name = file.split('/')[-1]
				batch_data.append({'filename': img_name})
				batch_files.append(('files', (img_name, open(file, 'rb'), 'image/jpeg')))
			batch_files.append(('data', ('', json.dumps(batch_data))))
			response = session.post(url, 
						auth=requests.auth.HTTPBasicAuth(self.api_key, ''), 
						files=batch_files)
			responses.append(response)
			batch_nb+=1
		result = read_prediction_response(responses, self.model_type)
		return result

	def predict_for_urls(self, urls):
		"""
		function to get prediction for a single image file from a trained model

		Parameters
		----------
		file_path: List[str]
			list of urls of the images you want to get predictions for

		Returns
		-------
		JSON repsonse of the predictions 
		"""
		
		response = self._predict_urls(urls)
		result = read_prediction_response([response], self.model_type)
		return result

		



import json
from xmljson import parker
from xml.etree.ElementTree import fromstring

def keep_keys(old_dict):
	new_dict = {}
	for key in old_dict:
		if key in ["object","segmented","size"]:
			new_dict[key]=old_dict[key]
	return new_dict

def annotation_xml_to_json(xml_file):
	f = open(xml_file, "rb")
	json_dict = keep_keys(parker.data(fromstring(f.read())))
	if not "object" in json_dict:
		json_dict = {"object": {}}
	json_output = json.dumps(json_dict["object"])
	f.close()
	return json_output

def process_annotations(image_name, annotation_path):
	if annotation_path.endswith('.json'):
		json_data = open(annotation_path, 'r').read()
	elif annotation_path.endswith('.xml'):
		json_data = annotation_xml_to_json(annotation_path)
	annotation = {"filename": image_name, 
		      "object": json.loads(json_data)}
	return annotation	

def read_prediction_response(response_list, model):
	if model == 'ImageCategorization' or model == 'MultiLabelClassification':
		file_key = 'file'
	else:
		file_key = 'input'
	if len(response_list) > 1:
		resp = [x.json() for x in response_list]
		res = [x['result'] for x in resp]
		file = [y[file_key] for x in res for y in x]
		pred = [y['prediction'] for x in res for y in x]
		final = dict(zip(file, pred))
	else:
		resp = response_list[0].json()
		res = resp['result']
		file = [x[file_key] for x in res]
		pred = [x['prediction'] for x in res]
		final = dict(zip(file, pred))
	return final



import nanonets.utils
from nanonets.model import Model
from nanonets.classification import ImageClassification
from nanonets.multilabel_classification import MultilabelClassification
from nanonets.object_detection import ObjectDetection
from nanonets.ocr import OCR 



from setuptools import setup, find_packages

NAME = "nanonets"
VERSION = "2.0.1"

REQUIRES = [line.strip() for line in open("requirements.txt").readlines()]

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

setup(
    name=NAME,
    version=VERSION,
    description="The Nanonets API",
    author_email="support@nanonets.com",
    url="https://nanonets.com",
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description_content_type='text/markdown',
    long_description=long_description,
)


import os
import json
import requests
from tqdm import tqdm

from nanonets.utils import *


class Model:

	def __init__(self, api_key, model_type):
		self.api_key = api_key
		self.model_type = model_type
		self.host = "https://app.nanonets.com/api/v2/"

	def _create_model(self, categories):
		"""
		function to create model given the categories and model type

		Parameters
		----------
		categories: List[str]
		    List of categories for our model

		Returns
		-------
		server response for the request to create model
		"""

		self.categories = categories
		url = self.host + self.model_type + '/Model/' 
		if not self.model_type == 'ImageCategorization':
			headers = {'Content-Type': "application/json"}
			payload = json.dumps({"categories": categories})
		else: 
			headers = {'Accept': "application/x-www-form-urlencoded"}
			payload = {"categories": categories}
		response =  requests.request("POST", url, 
					     headers=headers, 
					     data=payload,
					     auth=requests.auth.HTTPBasicAuth(self.api_key, ''))
		self.model_id = response.json()["model_id"]
		print("Your Model ID is: ", self.model_id)
		return response

	def _upload_image_file(self, img_path, annotation):
		"""
		function to upload a single file and it's labels for training to a model that 
		has been created

		Parameters
		----------
		img_path: str
		    path to the image we want to upload

		annotation: 
			List[str] of all labels associated with the image for MLC 
			or JSON file path for OD/OCR 

		Returns
		-------
		server response for the request for uploading image file
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/UploadFile/'
		img_name = img_path.split('/')[-1]
		params = {'file': open(image_path, 'rb'),
				  'modelId': ('', self.model_id),
				  'data' :('', annotation)}
		response = requests.post(url,
					files=params, 
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''))
		return response

	def _train(self):
		"""
		function to start training of a model that has been created

		Parameters
		----------
		categories: List[str]
		    List of categories for our model

		Returns
		-------
		server response for the request to initialize training
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/Train/'
		params = {'modelId': self.model_id}
		response = requests.request("POST", url,
					    params=params,
					    auth=requests.auth.HTTPBasicAuth(self.api_key, ''))
		return response

	def _check_model_state(self):
		"""
		function to check model state given you have the model id as a class attribute

		Parameters
		----------
		None

		Returns
		-------
		server response for the request to get model state
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id
		response = requests.request("GET", url,
					auth=requests.auth.HTTPBasicAuth(self.api_key,''))
		state = response.json()["state"]
		status = response.json()["status"]
		if state != 5:
			print("The model isn't ready yet, it's status is:", status)
			print("We will send you an email when the model is ready.")
		else:
			print("Model is ready for predictions.")
		return response

	def _predict_urls(self, image_urls):
		"""
		function to get predictions for images using urls

		Parameters
		----------
		image_urls: List[str]
			urls of the images you want to get predictions for

		Returns
		-------
		JSON repsonse of the predictions 
		"""

		url = self.host + self.model_type + '/Model/' + self.model_id + '/LabelUrls/'
		params = {'modelId': self.model_id, 'urls': image_urls}
		response = requests.post(url,
					auth=requests.auth.HTTPBasicAuth(self.api_key, ''),
					data=params) 
		return response
